@@ -363,7 +363,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 		 * which is array.size * raid_disks / ncopies;
 		 * .. but convert to sectors.
 		 */
-		int ncopies = (layout>>8) * (layout & 255);
+		int ncopies = ((layout>>8) & 255) * (layout & 255);
 		bitmapsize = (unsigned long long)size * raiddisks / ncopies * 2;
 /*		printf("bms=%llu as=%d rd=%d nc=%d\n", bitmapsize, size, raiddisks, ncopies);*/
 	} else
@@ -397,6 +397,13 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 
 	if (name == NULL || *name == 0) {
 		/* base name on mddev */
+		/*  /dev/md0 -> 0
+		 *  /dev/md_d0 -> d0
+		 *  /dev/md/1 -> 1
+		 *  /dev/md/d1 -> d1
+		 *  /dev/md/home -> home
+		 *  /dev/mdhome -> home
+		 */
 		name = strrchr(mddev, '/');
 		if (name) {
 			name++;
@@ -404,7 +411,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 			    strlen(name) > 4 &&
 			    isdigit(name[4]) &&
 			    (name-mddev) == 5 /* /dev/ */)
-				name += 4;
+				name += 3;
 			else if (strncmp(name, "md", 2)==0 &&
 				 strlen(name) > 2 &&
 				 isdigit(name[2]) &&
@@ -514,6 +521,7 @@ int Create(struct supertype *st, char *mddev, int mdfd,
 				fstat(fd, &stb);
 				disk.major = major(stb.st_rdev);
 				disk.minor = minor(stb.st_rdev);
+				remove_partitions(fd);
 				close(fd);
 			}
 			switch(pass){
