@@ -33,6 +33,7 @@ inline void sb_le_to_cpu(bitmap_super_t *sb)
 	sb->chunksize = __le32_to_cpu(sb->chunksize);
 	sb->daemon_sleep = __le32_to_cpu(sb->daemon_sleep);
 	sb->sync_size = __le64_to_cpu(sb->sync_size);
+	sb->write_behind = __le32_to_cpu(sb->write_behind);
 }
 
 inline void sb_cpu_to_le(bitmap_super_t *sb)
@@ -344,11 +345,12 @@ int CreateBitmap(char *filename, int force, char uuid[16],
 	if (chunksize == UnSet) {
 		/* We don't want more than 2^21 chunks, as 2^11 fill up one
 		 * 4K page (2 bytes per chunk), and 2^10 address of those
-		 * fill up a 4K indexing page.  2^20 might be safer...
+		 * fill up a 4K indexing page.  2^20 might be safer, especially
+		 * on 64bit hosts, so use that.
 		 */
 		chunksize = DEFAULT_BITMAP_CHUNK;
-		/* <<21 for 2^21 chunks, >>9 to convert bytes to sectors */
-		while (array_size > (chunksize << (21-9)))
+		/* <<20 for 2^20 chunks, >>9 to convert bytes to sectors */
+		while (array_size > (chunksize << (20-9)))
 			chunksize <<= 1;
 	}
 
@@ -389,6 +391,7 @@ int CreateBitmap(char *filename, int force, char uuid[16],
 	}
 	
 	rv = 0;
+	fflush(fp);
 	/* make the file be the right size (well, to the nearest byte) */
 	if (ftruncate(fileno(fp), filesize))
 		perror("ftrunace");
